/*
  Xiphos, a UCI chess engine
  Copyright (C) 2018 Milos Tatarevic

  Xiphos is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Xiphos is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <string.h>
#include "eval.h"
#include "game.h"

int pst_mid[P_LIMIT][BOARD_SIZE], pst_end[P_LIMIT][BOARD_SIZE];
const int adj_piece_value_mid[N_PIECES] = { -20, 0, 0, 0, 0, 0 };

const int pst_mid_base[N_PIECES][BOARD_SIZE] = {
  {
    0,   0,   0,   0,   0,   0,   0,   0,
    0,   0,   0,   0,   0,   0,   0,   0,
    0,   0,   0,   0,   0,   0,   0,   0,
    0,   0,   5,  10,  10,   0,  -5,  -5,
    0,   0,  10,  10,  10,   0,  -5,  -5,
    0,   0,  -5,  -5,  -5,  -5,   0,   0,
    0,  -5,  -5, -10, -10,   5,  15,   5,
    0,   0,   0,   0,   0,   0,   0,   0,
 }, {
  -30, -15,   0,   5,   5,   0, -15, -30,
  -20,   5,  20,  35,  35,  35,  10,  20,
  -15,  10,  25,  35,  35,  35,  35,  20,
  -10,  -5,  15,  20,  20,  20,  20,  20,
  -10,   0,   0,   5,  10,  10,  15,   0,
  -20, -10,   0,   5,  10,   0,   0, -20,
  -30, -20, -15,  -5,  -5, -15, -15,  -5,
  -45, -20, -30, -30, -30, -15, -30, -45,
 }, {
   -6,  -4,  -2,  -4,  -4,  -2,  -3,  -4,
   -2,  -2,   4,   4,   6,   6,   4,   2,
    1,   5,   4,   8,   9,   9,   9,   8,
    0,   3,   5,   5,   6,   5,   3,   1,
    0,   1,   4,   6,   4,   2,   3,   2,
    1,   2,   3,   3,   2,   0,   1,   0,
    2,   0,   2,  -2,   0,   0,   1,   1,
   -4,  -4,  -2,  -6,  -6,  -2,  -6,  -6,
 }, {
    0,   5,  10,  10,  10,  10,  10,  10,
   -5,   5,  10,  20,  20,  20,  20,  15,
   -5,   5,  10,  15,  15,  15,  15,  15,
   -5,   0,   5,  10,  10,   5,   5,   5,
  -10,   0,   5,  -5,  -5,  -5,   5,   0,
  -15,  -5,   0,  -5,  -5,  -5,   5,   0,
  -10,  -5,   0,  -5,  -5,   0,   0, -15,
  -10,  -5,   0,   5,   0,   5,   0, -10,
 }, {
   -5,   0,   0,   5,   5,  10,  10,  10,
    0,   5,  10,   5,   5,   5,  15,  20,
   -5,   5,   5,   0,   5,  15,   5,  10,
    0,   0,   5,   0,   5,   5,   5,   0,
   -5,   5,   5,   0,   5,   5,   5,   0,
   -5,   0,   5,   0,   0,   5,   5,   0,
   -5,   0,   0,   5,   0,  -5,  -5,   0,
  -10,  -5,  -5,   0,   0,  -5,  -5,  -5,
 }, {
  -20, -20, -20, -20, -20, -20, -20, -20,
  -20, -20, -20, -20, -20, -20, -20, -20,
  -20, -20, -20, -20, -20, -20, -20, -20,
  -20, -20, -20, -20, -20, -20, -20, -20,
  -20, -20, -20, -20, -20, -20, -20, -20,
  -20, -20, -20, -20, -20, -20, -20, -20,
    0,   0, -15, -15, -15, -15,   5,   0,
    5,  10,   0, -15,  -5, -15,  20,   5,
  }
};

const int pst_end_base[N_PIECES][BOARD_SIZE] = {
  {
    0,   0,   0,   0,   0,   0,   0,   0,
   35,  40,  35,  30,  30,  35,  40,  35,
   25,  30,  25,  15,  15,  25,  30,  25,
   10,  10,  10,   5,   5,  10,  10,  10,
    5,   5,   5,   0,   0,   5,   5,   5,
   -5,  -5,  -5,  -5,  -5,  -5,  -5,  -5,
    0,   0,   0,   0,   0,   0,   0,   0,
    0,   0,   0,   0,   0,   0,   0,   0,
 }, {
  -15,   0,   5,  10,  10,   5,   0, -15,
   -5,   5,  20,  20,  20,  20,   5,  -5,
    0,  10,  20,  25,  25,  20,  10,   0,
   -5,   5,  20,  30,  30,  20,   5,  -5,
   -5,   5,  15,  20,  20,  15,   5,  -5,
  -15,  -5,   0,  10,  10,   0,  -5, -15,
  -20, -15, -10,  -5,  -5, -10, -15, -20,
  -40, -30, -20, -10, -10, -20, -30, -40,
 }, {
    0,   1,   2,   2,   2,   2,   1,   0,
    1,   2,   2,   3,   3,   2,   2,   1,
    3,   4,   4,   5,   5,   4,   4,   3,
    2,   3,   4,   5,   5,   4,   3,   2,
    1,   3,   4,   5,   5,   4,   3,   1,
    0,   2,   3,   3,   3,   3,   2,   0,
   -2,  -1,   0,   1,   1,   0,  -1,  -2,
   -5,  -4,  -3,  -2,  -2,  -3,  -4,  -5,
 }, {
    5,   5,  10,  15,  15,  10,   5,   5,
   10,  15,  15,  15,  15,  15,  15,  10,
    5,  10,  10,  10,  10,  10,  10,   5,
    5,  10,   5,   5,   5,   5,  10,   5,
    5,  10,   5,   5,   5,   5,  10,   5,
    0,   5,   0,   0,   0,   0,   5,   0,
  -10,  -5,  -5,  -5,  -5,  -5,  -5, -10,
  -15, -10, -10, -10, -10, -10, -10, -15,
 }, {
    5,   5,  10,  15,  15,  10,   5,   5,
   10,  15,  20,  25,  25,  20,  15,  10,
   10,  15,  20,  25,  25,  20,  15,  10,
   10,  15,  15,  20,  20,  15,  15,  10,
    5,   5,  10,  10,  10,  10,   5,   5,
   -5,   0,   0,   0,   0,   0,   0,  -5,
  -15, -15, -10, -10, -10, -10, -15, -15,
  -25, -20, -20, -20, -20, -20, -20, -25,
 }, {
    0,  20,  30,  30,  30,  30,  20,   0,
    5,  30,  45,  40,  40,  45,  30,   5,
   10,  30,  40,  40,  40,  40,  30,  10,
   10,  20,  25,  30,  30,  25,  20,  10,
    5,  10,  15,  20,  20,  15,  10,   5,
   -5,   0,   5,  10,  10,   5,   0,  -5,
  -15, -10, -10,   0,   0, -10, -10, -15,
  -30, -20, -20, -15, -15, -20, -20, -30,
  }
};

void init_pst()
{
  int i, j, v;

  memset(pst_mid, 0, sizeof(pst_mid));
  memset(pst_end, 0, sizeof(pst_end));

  for (i = 0; i < N_PIECES; i ++)
  {
    for (j = 0; j < BOARD_SIZE; j ++)
    {
      v = pst_mid_base[i][j] + piece_value[i] + adj_piece_value_mid[i];
      pst_mid[i][j] = v;
      pst_mid[i | CHANGE_SIDE][j ^ 56] = v;

      v = pst_end_base[i][j] + piece_value[i];
      pst_end[i][j] = v;
      pst_end[i | CHANGE_SIDE][j ^ 56] = v;
    }
  }
}
